@@ -3,9 +3,9 @@
 
 #include <glib.h>
 #include <gdk/gdk.h>
-#include "panel-widget.h"
-#include "panel-gconf.h"
+
 #include "panel-enums.h"
+#include "panel-widget.h"
 
 G_BEGIN_DECLS
 
@@ -17,9 +17,10 @@ G_BEGIN_DECLS
 typedef struct {
 	PanelObjectType  type;
 	GtkWidget       *widget;
+	GSettings       *settings;
 
 	GtkWidget       *menu;
-	GtkWidget       *move_item;
+	GtkWidget       *edit_menu;
 	GList           *user_menu;
 
 	gpointer         data;
@@ -45,15 +46,12 @@ typedef struct {
 } AppletUserMenu;
 
 AppletInfo *panel_applet_register    (GtkWidget       *applet,
-				      gpointer         data,
-				      GDestroyNotify   data_destroy,
 				      PanelWidget     *panel,
-				      gboolean         locked,
-				      gint             pos,
-				      gboolean         exactpos,
 				      PanelObjectType  type,
-				      const char      *id);
-void panel_applet_stop_loading (const char *id);
+				      const char      *id,
+				      GSettings       *settings,
+				      gpointer         data,
+				      GDestroyNotify   data_destroy);
 
 const char *panel_applet_get_id           (AppletInfo      *info);
 const char *panel_applet_get_id_by_widget (GtkWidget       *widget);
@@ -64,16 +62,6 @@ GSList     *panel_applet_list_applets (void);
 
 void        panel_applet_clean        (AppletInfo    *info);
 
-void panel_applet_queue_applet_to_load (const char      *id,
-					PanelObjectType  type,
-					const char      *toplevel_id,
-					int              position,
-					gboolean         right_stick,
-					gboolean         locked);
-void panel_applet_load_queued_applets  (gboolean initial_load);
-gboolean panel_applet_on_load_queue    (const char *id);
-
-
 void            panel_applet_add_callback    (AppletInfo          *info,
 					      const gchar         *callback_name,
 					      const gchar         *stock_item,
@@ -89,22 +77,16 @@ void        panel_applet_save_position           (AppletInfo *applet_info,
 						  gboolean    immediate);
 
 int         panel_applet_get_position    (AppletInfo *applet);
+GSettings  *panel_applet_get_settings    (AppletInfo *applet);
 
 /* True if all the keys relevant to moving are writable
    (position, toplevel_id, panel_right_stick) */
 gboolean    panel_applet_can_freely_move (AppletInfo *applet);
 
-/* True if the locked flag is writable */
-gboolean    panel_applet_lockable (AppletInfo *applet);
-
-GtkWidget  *panel_applet_create_menu (AppletInfo *info);
-
 void        panel_applet_menu_set_recurse (GtkMenu     *menu,
 					   const gchar *key,
 					   gpointer     data);
 
-gboolean    panel_applet_toggle_locked  (AppletInfo *info);
-
 void panel_applet_position_menu (GtkMenu   *menu,
 				 int       *x,
 				 int       *y,
