@@ -34,7 +34,6 @@
 
 #include "launcher.h"
 #include "panel.h"
-#include "drawer.h"
 #include "panel-applets-manager.h"
 #include "panel-applet-frame.h"
 #include "panel-action-button.h"
@@ -43,9 +42,9 @@
 #include "panel-toplevel.h"
 #include "panel-menu-button.h"
 #include "panel-globals.h"
+#include "panel-layout.h"
 #include "panel-lockdown.h"
 #include "panel-util.h"
-#include "panel-profile.h"
 #include "panel-addto.h"
 #include "panel-icon-names.h"
 
@@ -87,8 +86,7 @@ typedef enum {
 	PANEL_ADDTO_LAUNCHER_NEW,
 	PANEL_ADDTO_MENU,
 	PANEL_ADDTO_MENUBAR,
-	PANEL_ADDTO_SEPARATOR,
-	PANEL_ADDTO_DRAWER
+	PANEL_ADDTO_SEPARATOR
 } PanelAddtoItemType;
 
 typedef struct {
@@ -168,17 +166,6 @@ static PanelAddtoItemInfo internal_addto_items [] = {
 	  NULL,
 	  NULL,
 	  "SEPARATOR:NEW",
-	  TRUE },
-
-	{ PANEL_ADDTO_DRAWER,
-	  N_("Drawer"),
-	  N_("A pop out drawer to store other items in"),
-	  PANEL_ICON_DRAWER,
-	  PANEL_ACTION_NONE,
-	  NULL,
-	  NULL,
-	  NULL,
-	  "DRAWER:NEW",
 	  TRUE }
 };
 
@@ -325,7 +312,7 @@ panel_addto_setup_drag (GtkTreeView          *tree_view,
 			const GtkTargetEntry *target,
 			const char           *text)
 {
-	if (!text || panel_lockdown_get_locked_down ())
+	if (!text || panel_lockdown_get_panels_locked_down_s ())
 		return;
 	
 	gtk_tree_view_enable_model_drag_source (GTK_TREE_VIEW (tree_view),
@@ -400,7 +387,9 @@ panel_addto_query_applets (GSList *list)
 		description = panel_applet_info_get_description (info);
 		icon = panel_applet_info_get_icon (info);
 
-		if (!name || panel_lockdown_is_applet_disabled (iid)) {
+		if (!name ||
+		    panel_lockdown_is_applet_disabled (panel_lockdown_get (),
+						       iid)) {
 			continue;
 		}
 
@@ -477,7 +466,7 @@ panel_addto_append_special_applets (PanelAddtoDialog *dialog,
 		}
 
 		if (special_addto_items [i].type == PANEL_ADDTO_LAUNCHER_NEW
-		    && panel_lockdown_get_disable_command_line ())
+		    && panel_lockdown_get_disable_command_line_s ())
 			continue;
 		
 		panel_addto_append_item (dialog, model, &special_addto_items [i]);
@@ -495,7 +484,7 @@ panel_addto_make_applet_model (PanelAddtoDialog *dialog)
 	if (dialog->filter_applet_model != NULL)
 		return;
 
-	if (panel_profile_id_lists_are_writable ()) {
+	if (panel_layout_is_writable ()) {
 		dialog->applet_list = panel_addto_query_applets (dialog->applet_list);
 		dialog->applet_list = panel_addto_prepend_internal_applets (dialog->applet_list);
 	}
@@ -509,7 +498,7 @@ panel_addto_make_applet_model (PanelAddtoDialog *dialog)
 				    G_TYPE_POINTER,
 				    G_TYPE_STRING);
 
-	if (panel_profile_id_lists_are_writable ()) {
+	if (panel_layout_is_writable ()) {
 		panel_addto_append_special_applets (dialog, model);
 		if (dialog->applet_list)
 			panel_addto_append_item (dialog, model, NULL);
@@ -787,14 +776,13 @@ panel_addto_add_item (PanelAddtoDialog   *dialog,
 		break;
 	case PANEL_ADDTO_LAUNCHER_NEW:
 		ask_about_launcher (NULL, dialog->panel_widget,
-				    dialog->insertion_position, FALSE);
+				    dialog->insertion_position);
 		break;
 	case PANEL_ADDTO_MENU:
 		panel_menu_button_create (dialog->panel_widget->toplevel,
 					  dialog->insertion_position,
 					  item_info->menu_filename,
 					  item_info->menu_path,
-					  item_info->menu_path != NULL,
 					  item_info->name);
 		break;
 	case PANEL_ADDTO_MENUBAR:
@@ -805,11 +793,6 @@ panel_addto_add_item (PanelAddtoDialog   *dialog,
 		panel_separator_create (dialog->panel_widget->toplevel,
 					dialog->insertion_position);
 		break;
-	case PANEL_ADDTO_DRAWER:
-		panel_drawer_create (dialog->panel_widget->toplevel,
-				     dialog->insertion_position,
-				     NULL, FALSE, NULL);
-		break;
 	}
 }
 
@@ -973,13 +956,11 @@ panel_addto_dialog_free_application_list (GSList *application_list)
 static void
 panel_addto_dialog_free (PanelAddtoDialog *dialog)
 {
-	GConfClient *client;
-	GSList      *item;
-
-	client = panel_gconf_get_client ();
+	GSList *item;
 
 	if (dialog->name_notify)
-		gconf_client_notify_remove (client, dialog->name_notify);
+		g_signal_handler_disconnect (dialog->panel_widget->toplevel,
+					     dialog->name_notify);
 	dialog->name_notify = 0;
 
 	if (dialog->search_text)
@@ -1032,57 +1013,34 @@ panel_addto_dialog_free (PanelAddtoDialog *dialog)
 }
 
 static void
-panel_addto_name_change (PanelAddtoDialog *dialog,
-			 const char       *name)
+panel_addto_name_change (PanelAddtoDialog *dialog)
 {
-	char *title;
-	char *label;
+	const char *name;
+	char       *label;
 
+	name = panel_toplevel_get_name (dialog->panel_widget->toplevel);
 	label = NULL;
 
 	if (!PANEL_GLIB_STR_EMPTY (name))
 		label = g_strdup_printf (_("Find an _item to add to \"%s\":"),
 					 name);
 
-	if (panel_toplevel_get_is_attached (dialog->panel_widget->toplevel)) {
-		title = g_strdup_printf (_("Add to Drawer"));
-		if (label == NULL)
-			label = g_strdup (_("Find an _item to add to the drawer:"));
-	} else {
-		title = g_strdup_printf (_("Add to Panel"));
-		if (label == NULL)
-			label = g_strdup (_("Find an _item to add to the panel:"));
-	}
+	if (label == NULL)
+		label = g_strdup (_("Find an _item to add to the panel:"));
 
-	gtk_window_set_title (GTK_WINDOW (dialog->addto_dialog), title);
-	g_free (title);
+	gtk_window_set_title (GTK_WINDOW (dialog->addto_dialog),
+			      _("Add to Panel"));
 
 	gtk_label_set_text_with_mnemonic (GTK_LABEL (dialog->label), label);
 	g_free (label);
 }
 
 static void
-panel_addto_name_notify (GConfClient      *client,
-			 guint             cnxn_id,
-			 GConfEntry       *entry,
+panel_addto_name_notify (GObject          *gobject,
+			 GParamSpec       *pspec,
 			 PanelAddtoDialog *dialog)
 {
-	GConfValue *value;
-	const char *key;
-	const char *text = NULL;
-
-	key = panel_gconf_basename (gconf_entry_get_key (entry));
-
-	if (strcmp (key, "name"))
-		return;
-
-	value = gconf_entry_get_value (entry);
-
-	if (value && value->type == GCONF_VALUE_STRING)
-		text = gconf_value_get_string (value);
-
-	if (text)
-		panel_addto_name_change (dialog, text);
+	panel_addto_name_change (dialog);
 }
 
 static gboolean
@@ -1203,7 +1161,7 @@ panel_addto_selection_changed (GtkTreeSelection *selection,
 				  TRUE);
 
 	/* only allow dragging applets if we can add applets */
-	if (panel_profile_id_lists_are_writable ()) {
+	if (panel_layout_is_writable ()) {
 		switch (data->type) {
 		case PANEL_ADDTO_LAUNCHER:
 			panel_addto_setup_launcher_drag (GTK_TREE_VIEW (dialog->tree_view),
@@ -1285,12 +1243,10 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 
 	dialog->panel_widget = panel_widget;
 	dialog->name_notify =
-		panel_profile_toplevel_notify_add (
-			dialog->panel_widget->toplevel,
-			"name",
-			(GConfClientNotifyFunc) panel_addto_name_notify,
-			dialog);
-
+		g_signal_connect (dialog->panel_widget->toplevel,
+				  "notify::name",
+				  G_CALLBACK (panel_addto_name_notify),
+				  dialog);
 
 	dialog->addto_dialog = gtk_dialog_new ();
 	gtk_dialog_add_button (GTK_DIALOG (dialog->addto_dialog),
@@ -1408,8 +1364,7 @@ panel_addto_dialog_new (PanelWidget *panel_widget)
 	panel_widget_register_open_dialog (panel_widget,
 					   dialog->addto_dialog);
 
-	panel_addto_name_change (dialog,
-				 panel_toplevel_get_name (dialog->panel_widget->toplevel));
+	panel_addto_name_change (dialog);
 
 	return dialog;
 }
